#! /usr/bin/perl
#
# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


use strict;
use warnings FATAL => 'all';
use diagnostics;

require "html.pl";
require "db.pl";
require "dbops.pl";
require "url.pl";
require "common.pl";
require "lang.pl";
require "urllist.pl";



&main();
exit(0);


sub main
{
	&HTML::print_content();
	&HTML::print_header();

	my $user = &HTML::get("user", -1);
	my $seek = &HTML::get("seek", -1);
	my $n = &HTML::get("n", -1);

	if ($seek == -1) {
		&summary($user);
	} else {
		&url_hist($user, $n, $seek);
	}

	&HTML::print_footer(&DB::process_time());
}


# 0: user
sub summary
{
	my ($id) = @_;

	my ($nick) = &DBOps::get_nick_name($id);
	(defined($nick)) || &HTML::bad_param("bad user");

	printf "<h1>" . &Lang::get("user-summary") . "</h1>\n", $nick;

	my @score = &get_score($id, 10);
	($#score != -1) && &print_score(@score);

	my $r = &urls($id, 0, 8);
	if ($r > 0) {
		&urls($id, 1, 4);
	} else {
		printf "<p>" . &Lang::get("user-no-urls") . "</p>\n", $nick;
	}
}


# 0: user
# 1: n
# 2: seek
sub url_hist
{
	my ($id, $n, $seek) = @_;

	my $nick = &DBOps::get_nick_name($id);
	(defined($nick)) || &HTML::bad_param("bad user");

	printf "<h1>" . &Lang::get("user-hist-for") . "</h1>\n", $nick;

	&urls($id, 0, $n, $seek);
}


# 0: csv
sub print_score
{
	my $l_chan = &Lang::get("col-channel");
	my $l_fame = &Lang::get("col-fame");
	my $l_shame = &Lang::get("col-shame");
	my $l_urls = &Lang::get("col-urls");
	my $l_ratio = &Lang::get("col-points-ratio");
	my $l_wanha = &Lang::get("col-wanha");
	my $l_cheat = &Lang::get("col-cheat");

	print <<EOF;
<table class="perchan">
	<tr class="ul">
		<th>$l_chan</th>
		<th>$l_urls</th>
		<th>$l_fame</th>
		<th>$l_ratio</th>
		<th>$l_shame</th>
		<th>$l_ratio</th>
		<th>$l_wanha</th>
		<th>$l_cheat</th>
	</tr>
EOF

	my $t_fame = 0;
	my $t_shame = 0;
	my $t_wanha = 0;
	my $t_cheat = 0;
	my $t_urls = 0;
	my $i = 0;
	foreach (@_) {
		my ($count, $id, $fame, $shame, $wanha, $cheat, $name) =
				split(/,/, $_, 7);
		my $a_chan = &URL::chan($id);
		my $class = $i % 2 == 0 ? "zebra_c" : "center";

		my $total = $count + $shame;
		my $f_ratio = sprintf("%.1f", $fame / $total * 100);
		my $s_ratio = sprintf("%.1f", $shame / $total * 100);

		print <<EOF;
	<tr class="$class">
		<td class="left"><a href="$a_chan">$name</a></td>
		<td>$total</td>
		<td>$fame</td>
		<td>$f_ratio%</td>
		<td>$shame</td>
		<td>$s_ratio%</td>
		<td>$wanha</td>
		<td>$cheat</td>
	</tr>
EOF
		$t_fame += $fame;
		$t_shame += $shame;
		$t_urls += $total;
		$t_wanha += $wanha;
		$t_cheat += $cheat;
		$i++;
	}

	my $l_total = &Lang::get("col-total");
	print <<EOF;
	<tr class="sum">
		<td>$l_total</td>
		<td class="center">$t_urls</td>
		<td class="center">$t_fame</td>
		<td class="center"></td>
		<td class="center">$t_shame</td>
		<td class="center"></td>
		<td class="center">$t_wanha</td>
		<td class="center">$t_cheat</td>
	</tr>
</table>
EOF
}


# 0: user
# 1: mode, 0 == latest, 1 == top-n
# 2: n
# 3: seek
# r: 0 if urls, 1 if no urls
sub urls
{
	my ($user_id, $mode, $n, $seek) = @_;
	my @urls;
	my @images;
	my $has_more;

	$n = 25 if ($n > 25);

	# get one extra so that we know if there's more
	@urls = &get_urls($user_id, $mode, $n + 1, $seek);
	($#urls == -1 && defined($seek)) && &HTML::bad_param("bad seek");
	($#urls == -1) && return 0;
	if ($#urls == $n) {
		$has_more = 1;
		pop @urls;
	} else {
		$has_more = 0;
	}

	if (! defined($seek)) {
		my $fmt = &Lang::get(($mode == 0) ?
				"user-n-latest-urls" : "user-n-best-urls");
		printf "<h2>" . $fmt . "</h2>\n", $n;
	} else {
		if ($mode == 0) {
			my ($first) = $urls[0] =~ /^(\d+),/;
			my ($last) = $urls[$#urls] =~ /^(\d+),/;
			printf "<h2>" . &Lang::get("user-urls-range") .
					"</h2>\n", $first, $last;
		} else {
			my $first = $seek + 1;
			my $last = $n + $seek;
			printf "<h2>" . &Lang::get("user-pop-urls-range") .
					"</h2>\n", $first, $last;
		}
	}

	@images = &URLList::print(2, defined($seek) ? 1 : 0, @urls);

	($mode == 1) && return 1;
	my $l;
	print "<p>" . &Lang::get("link-go-to") . "\n";
	if (! defined($seek)) {
		my $a_user = &URL::user($user_id, 25, $n);
		my $l = &Lang::get("link-older-urls");
		print "\t<a href=\"$a_user\">\u$l</a>\n";
	} else {
		my $a_user = &URL::user($user_id);
		my $l = &Lang::get("link-user-summary");
		print "\t<a href=\"$a_user\">\u$l</a>";
		if ($seek > 0) {
			my $tseek = $seek - $n;
			$tseek = 0 if ($tseek < 0);
			my $a_user = &URL::user($user_id, $n, $tseek);
			print ",\n\t<a href=\"$a_user\">" .
					&Lang::get("link-newer") . "</a>";
		}
		if ($has_more == 1) {
			my $tseek = $seek + $n;
			my $a_user = &URL::user($user_id, $n, $tseek);
			print ",\n\t<a href=\"$a_user\">" .
					&Lang::get("link-older") . "</a>";
		}
	}
	print "</p>\n";

	&Common::link_images(@images);

	return 1;
}


# 0: user
# 1: mode, 0 == latest, 1 == top-n
# 2: n
# 3: seek
# r: csv (url_id, url_time, url_count, chan_id, chan_name, url)
sub get_urls
{
	my ($user_id, $mode, $n, $seek) = @_;
	my ($query, $result);
	my @ret;

	my $order;
	if ($mode == 0) {
		$order = "urls.time";
	} elsif ($mode == 1) {
		$order = "urls.count";
	} else {
		&HTML::bad_param("mode");
	}

	$query = "SELECT urls.url, urls.id, " .
				&DBOps::get_date_fmt("urls.time") . ", " .
				"urls.count, chans.id, chans.chan " .
			"FROM urls, chans " .
			"WHERE urls.nick = $user_id " .
				"AND urls.chan = chans.id " .
			"ORDER BY $order DESC " .
			"LIMIT $n";
	$query .= " OFFSET $seek" if (defined($seek));

	$result = &DB::query($query);
	($result->ntuples == 0) && return ();

	while (my ($url_url, $url_id, $url_time, $url_count,
			$chan_id, $chan_name) = $result->fetchrow) {
		$chan_name =~ s/,/./g;
		push(@ret, "$url_id,$url_time,$url_count," .
				"$chan_id,$chan_name,$url_url");
	}
	return @ret;
}


# 0: nick id
# 1: n
# r: csv (count, id, fame, shame, wanha, cheat, name)
sub get_score
{
	my ($id, $n) = @_;

	my $query = "SELECT chans.id, chans.chan, " .
				"sb.fame, sb.shame, sb.wanha, sb.cheat " .
			"FROM chans, scoreboard AS sb " .
			"WHERE sb.nick = $id AND sb.chan = chans.id " .
			"ORDER BY sb.fame, sb.shame DESC ".
			"LIMIT $n";
	my $result = &DB::query($query);
	if ($result->ntuples == 0) {
		return;
	}

	my @ret;
	while (my ($id, $name, $fame, $shame, $wanha, $cheat) =
			$result->fetchrow) {
		push(@ret, "$id,$fame,$shame,$wanha,$cheat,$name");
	}

	foreach (@ret) {
		my ($chan) = split(/,/);
		my $c;
		$query = "SELECT COUNT(id) FROM urls " .
				"WHERE nick = $id AND chan = $chan";
		$result = &DB::query($query);
		($c) = $result->fetchrow;
		s/^/$c,/;
	}

	return @ret;
}
